<?php

namespace Service;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Cron\CronExpression;
use Zeedhi\Framework\DataSource\Manager\Mongo\Types\Type;

class GoalNotifications {

    protected $dataSourceName = 'goalNotifications';
    /** @var \Service\Manager\Mongo\Manager $organizationManager */
    protected $mongoManager;
    /** @var \Service $dataSource */
    protected $dataSourceService;
    /** @var \Service $goals */
    protected $goalsService;
    /** @var \Service $notification */
    protected $notificationsService;
    /** @var \Service\WidgetData $widget */
    protected $widgetDataService;
    /** @var \Service $utilities */
    protected $utilities;

    protected $ERROR_CODE = 'error';

    public function __construct (Manager $mongoManager, $dataSourceService, $goalsService,
                                 $notificationsService, $widgetDataService, $utilities){
        $this->mongoManager = $mongoManager;
        $this->dataSourceService = $dataSourceService;
        $this->goalsService = $goalsService;
        $this->notificationsService = $notificationsService;
        $this->widgetDataService = $widgetDataService;
        $this->utilities = $utilities;
    }

    public function saveNotification($notification) {
        $dataSet = new DataSet($this->dataSourceName, array($notification));
        return $this->mongoManager->persist($dataSet);
    }

    public function findBy($propertyName, $propertyValue, $operator = '=') {
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('isActive', "=", true);
        $filterCriteria->addCondition($propertyName, $operator, $propertyValue);
        return $this->mongoManager->findBy($filterCriteria)->getRows();
    }

    public function findNext($params, $decode = true) { 
        return $this->mongoManager->findNext($this->dataSourceName, $params, $decode);
    }

    public function countNext($params, $decode = true) {
        return $this->mongoManager->countNext($this->dataSourceName, $params, $decode);
    }

    public function getGoalNotificationByDataSourceId($dataSourceId) {
        return $this->findBy('dataSourceId', $dataSourceId, "=");
    }

    public function getGoalNotificationByGoalId($goalId) {
        return $this->findBy('goalId', $goalId, "=");
    }

    public function delete($goalNotification) {
        $dataSet = new DataSet($this->dataSourceName, $goalNotification);
        return $this->mongoManager->delete($dataSet);
    }

    public function checkGoalNotifications($dataSourceId) {
        $goalNotifications = $this->getGoalNotificationByDataSourceId($dataSourceId);
        foreach ($goalNotifications as $goalNotification) {
            $this->checkNotificationsByGoalNotification($goalNotification, $dataSourceId);
        }
    }

    protected function checkNotificationsByGoalNotification($goalNotification, $dataSourceId) {
        $goal = $this->goalsService->getGoalById($goalNotification['goalId']);
        $period = isset($goal['viewPeriod']) ? $goal['viewPeriod'] : null;
        $structureId = isset($goal['filteredStructureId']) ? $goal['filteredStructureId'] : null;
        $widgetData = $this->getWidgetData($goal['viewId'], $period, $structureId);
        $goalVerification = $this->checkGoal($goal, $widgetData, $goalNotification);
        if($goalVerification !== $this->ERROR_CODE) {
            $isGoalAchieved = $goalVerification['isGoalAchieved'];
            $fieldsEvaluation = $goalVerification['fieldsEvaluation'];

            if($this->validateRunNotificationConditions($goalNotification, $isGoalAchieved)) {
                $this->sendMailNotification($dataSourceId, $goalNotification, $goal, $goalVerification);
            }
        }
    }

    protected function getWidgetData($viewId, $period, $structureId = "") {
        return $this->widgetDataService->fetchWidgetData($viewId, array($structureId), null, $period);
    }

    protected function checkGoal($goalInfo, $widgetData, $goalNotification = null) {
        $goalsSentenceDecoded = $this->getGoalParams($goalInfo['goal'], $widgetData);
        $goal = $goalsSentenceDecoded['goal']['params'];
        $fieldsEvaluation = $goalsSentenceDecoded['goal']['fieldsEvaluation'];
        $goalConditions = $goalsSentenceDecoded['conditions'];
        
        $isGoalAchieved = $this->evaluateSentence($goal, $widgetData, $goalConditions, $goalNotification);

        if($isGoalAchieved === $this->ERROR_CODE) {
            return $this->ERROR_CODE;
        }

        return array(
            'isGoalAchieved' => $isGoalAchieved,
            'fieldsEvaluation' => $fieldsEvaluation
        );
    }

    protected function getGoalParams($goalSentence, $widgetData) {
        $goalConditions = explode("WHEN", $goalSentence);
        $conditions = isset($goalConditions[1]) ? $this->decodeGoalParamsSentence($goalConditions[1])['params'] : null;
        $goal = $this->decodeGoalParamsSentence($goalConditions[0], $widgetData, $conditions);
        return array(
            'goal' => $goal,
            'conditions' => $conditions
        );
    }

    protected function decodeGoalParamsSentence($params, $widgetData = null, $conditions = null) {
        $paramsConcated = '';
        $fieldsEvaluation = '';
        $explodedParams = explode(" ", $params);

        foreach ($explodedParams as &$param) {
            if(substr($param, 0, 1) === '$') {
                $paramName = str_replace('$', '', $param);
                $param = '$data["'.$paramName.'"]';

                if($conditions) {
                    $paramValue = $this->evaluateSentence($param, $widgetData, $conditions);
                    $fieldsEvaluation = $fieldsEvaluation . $paramName . ' = ' . $paramValue . "\n";
                }
            }

            $paramsConcated = $paramsConcated . $param;
        }

        return array(
            'params' => $paramsConcated,
            'fieldsEvaluation' => $fieldsEvaluation
        );
    }

    protected function evaluateSentence($sentence, $widgetData, $conditions, $goalNotification = null) {
        $evaluation = null;

        if(is_array($widgetData)) {
            if($conditions) {
                foreach ($widgetData as $data) {
                    if(eval('return '.$conditions.';')) {
                        $evaluation = eval('return '.$sentence.';');
                    }
                }
            } else {
                $evaluation = $this->ERROR_CODE;
            }
        } else {
            $evaluation = eval('return '.$sentence.';');
        }
        return $evaluation;
    }

    protected function validateRunNotificationConditions($notificationInfo, $isGoalAchieved) {
        $notifyCondition = $notificationInfo['notifyCondition'];

        switch ($notifyCondition) {
            case 'goalIsSatisfactory':
                return $isGoalAchieved;
            case 'goalIsNotSatisfactory':
                return !$isGoalAchieved;
            case 'alwaysOnCollect':
                return true;
            case 'periodically':
                if(isset($notificationInfo['nextNotifyDate'])) {
                    $now = $this->utilities->getTimezonedNow();
                    return $now >= $notificationInfo['nextNotifyDate'];
                } else {
                    return true;
                }
            default:
                return false;
        }
    }

    protected function sendMailNotification($dataSourceId, $goalNotification, $goal, $goalVerification) {
        $mailData = $this->buildMailData($dataSourceId, $goal, $goalNotification, $goalVerification);
        if($mailData) {
            $this->notificationsService->sendMailNotification($mailData, 'goalNotifications');
            $this->endNotificationEvent($goalNotification);
        }
    }

    protected function buildMailData($dataSourceId, $goal, $goalNotification, $goalVerification) {
        $dataSourceLastUpdate = $this->getDataSourceLastUpdate($dataSourceId);
        switch ($goalNotification["notifyCondition"]) {
            case 'goalIsSatisfactory':
                $notifyConditionText = "satisfizer a condição da meta <i>%label_goal%</i>.";
            break;
            case 'goalIsNotSatisfactory':
                $notifyConditionText = "não satisfizer a condição da meta <i>%label_goal%</i>.";
            break;
            default:
                $notifyConditionText = "realizar importação de dados.";
            break;
        }

        return array(
            'to' => $goalNotification['receiversMails'],
            'labelView' => isset($goal['viewLabel']) ? $goal['viewLabel'] : null,
            'dataSource' => isset($goal['dataSourceName']) ? $goal['dataSourceName'] : null,
            'lastUpdate' => $dataSourceLastUpdate,
            'labelGoal' => $goal['name'],
            'goal' => $goal['goal'],
            'value' => $goalVerification['fieldsEvaluation'],
            'userResponsible' => $goalNotification['createdBy'],
            'notificationText' => $goalNotification['notificationText'],
            'notifyCondition' => $notifyConditionText
        );
    }

    protected function getDataSourceLastUpdate($dataSourceId) {
        $now = $this->utilities->getTimezonedNow();
        $dataSource = $this->dataSourceService->findById($dataSourceId)[0];
        $lastUpdate = isset($dataSource['lastUpdate']) ? $dataSource['lastUpdate'] : $now;
        $lastUpdate = $lastUpdate->format("d/m/Y H:i");
        return $lastUpdate;
    }

    protected function endNotificationEvent($goalNotification) {
        $goalNotification = $this->setNotificationNextDate($goalNotification);
        $this->saveNotification($goalNotification);
    }

    public function setNotificationNextDate($goalNotification) {
        if($goalNotification['notifyCondition'] === 'periodically' && isset($goalNotification['scheduleExp'])) {
            $cron = CronExpression::factory($goalNotification['scheduleExp']);
            $nexDate = $cron->getNextRunDate();
            $goalNotification['nextNotifyDate'] = Type::getType('mongoDate')->convertToDatabaseValue($nexDate);
        } else if(isset($goalNotification['nextNotifyDate'])) {
            unset($goalNotification['nextNotifyDate']);
        }
        
        return $goalNotification;
    }
}