<?php

namespace tests\Service\Manager\Mongo;

use Service\Manager\Mongo\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\Mongo\Mongo;
use Zeedhi\Framework\Util\NameProvider;
use Zeedhi\Zmart\QueryBuilders\Mongo\QueryBuilder;

class ManagerTest extends \PHPUnit_Framework_TestCase {

	const MONGO_DB_NAME = 'ZeedhiKPIUnitTest';
	const MONGO_PORT = '27017';
	const MONGO_IP = '192.168.122.55';
	const DATASOURCE_NAME = 'widget';
	/** @var Manager */
	protected $manager;
	/** @var Mongo */
	protected static $mongoDB;

	public static function setUpBeforeClass() {
		if (!extension_loaded('mongo')) {
			self::markTestSkipped('The ' . __CLASS__ . ' requires the use of mongo extension.');
		}
		self::$mongoDB = new Mongo(self::MONGO_IP, self::MONGO_PORT, self::MONGO_DB_NAME);
	}

	public function setUp() {
		$ds = DIRECTORY_SEPARATOR;
		$slashes = $ds . '..' . $ds . '..' . $ds . '..' . $ds;
		$dataSourceDirectory = realpath(__DIR__) . $slashes . 'mocks' . $ds . 'gen' . $ds . 'datasources';
		$nameProvider = new NameProvider($dataSourceDirectory, "\\Model", array(), false);
		$this->manager = new Manager(self::$mongoDB, $nameProvider, new QueryBuilder());
	}

	public function testPersistInsert() {
		$rows = array(
			array(
				'id' => 1,
				'name' => 'widget1',
				'template' => 'widget/form.html'
			),
			array(
				'id' => 2,
				'name' => 'widget2',
				'template' => 'widget/form.html'
			),
			array(
				'id' => 3,
				'name' => 'widget3',
				'template' => 'widget/form.html'
			),
		);
		$dataSet = new DataSet(self::DATASOURCE_NAME, $rows);
		$persistedRows = $this->manager->persist($dataSet);
		$this->assertCount(3, $persistedRows);
	}

	/** @depends testPersistInsert */
	public function testFindByAll() {
		$widgetColumns = array('id', 'name', 'template', '_id');
		/** @var DataSet $dataSet */
		$dataSet = $this->manager->findBy(new FilterCriteria(self::DATASOURCE_NAME));
		$this->assertInstanceOf("\\Zeedhi\\Framework\\DataSource\\DataSet", $dataSet, "Return must be a instance of DataSet.");
		$this->assertCount(3, $dataSet->getRows());
		foreach ($dataSet->getRows() as $row) {
			$this->assertTrue(is_array($row));
			foreach ($row as $column => $value) {
				$this->assertContains($column, $widgetColumns, "Column must be one of know columns");
				$this->assertNotNull($value, "Value of columns must be not null");
			}
		}
	}

	/** @depends testPersistInsert */
	public function testFindByWithFilter() {
		$filterCriteria = new FilterCriteria(self::DATASOURCE_NAME);
		$filterCriteria->addCondition("name", "widget1");
		/** @var DataSet $dataSet */
		$dataSet = $this->manager->findBy($filterCriteria);
		$this->assertInstanceOf("\\Zeedhi\\Framework\\DataSource\\DataSet", $dataSet, "Return must be a instance of DataSet.");
		$this->assertCount(1, $dataSet->getRows(), "Should return 1 row.");
		foreach ($dataSet->getRows() as $row) {
			$this->assertEquals("widget1", $row["name"], "Widget retrieved must be name 'widget1'.");
		}
	}

	/** @depends testPersistInsert */
	public function testPersistUpdate() {
		$filterCriteria = new FilterCriteria(self::DATASOURCE_NAME);
		$filterCriteria->addCondition("name", "widget2");
		/** @var DataSet $dataSet */
		$dataSet = $this->manager->findBy($filterCriteria);
		$row = current($dataSet->getRows());
		$row['name'] = 'widgetUpdated';
		$row['template'] = 'widget/grid.html';
		$dataSet = new DataSet($dataSet->getDataSourceName(), array($row));

		$nrPersistedRows = $this->manager->persist($dataSet);
		$this->assertCount(1, $nrPersistedRows);
	}

	/** @depends testPersistInsert */
	public function testDelete() {
		$filterCriteria = new FilterCriteria(self::DATASOURCE_NAME);
		$filterCriteria->addCondition("name", "widget1");
		/** @var DataSet $dataSet */
		$dataSet = $this->manager->findBy($filterCriteria);

		$deletedRows = $this->manager->delete($dataSet);
		$this->assertCount(1, $deletedRows, "Number of deleted rows must be 1.");
		$dataSet = $this->manager->findBy($filterCriteria);
		$this->assertCount(0, $dataSet->getRows());
	}


	public static function tearDownAfterClass() {
		if (self::$mongoDB) {
			self::$mongoDB->dropCollection(self::DATASOURCE_NAME);
		}
	}

}
